@@ -9,6 +9,8 @@ open Indent
 open Atd.Ast
 open Mapping
 
+let target : Ocaml.target = Json
+
 (*
   OCaml code generator (json readers and writers)
 *)
@@ -98,53 +100,6 @@ val %s_of_string :%s
     Ox_emit.maybe_write_creator_intf ~with_create deref buf x
   )
 
-let is_json_string deref x =
-  (*
-    Calling 'unwrap' allows us to ignore 'wrap' constructors
-    and determine that the JSON representation is a string.
-    This assumes that no '<json>' annotation imposes
-    another representation for the JSON string.
-  *)
-  match Mapping.unwrap deref x with
-  | String _ -> true
-  | _ -> false (* or maybe we just don't know *)
-
-let get_assoc_type deref loc x =
-  match deref x with
-  | Tuple (_, [| k; v |], Ocaml.Repr.Tuple, Json.Tuple) ->
-      if not (is_json_string deref k.cel_value) then
-        Error.error loc "Due to <json repr=\"object\"> keys must be strings";
-      (k.cel_value, v.cel_value)
-  | _ ->
-      Error.error loc "Expected due to <json repr=\"object\">: (string * _) list"
-
-let get_fields p a =
-  List.map (
-    fun x ->
-      let ocamlf, jsonf =
-        match x.f_arepr, x.f_brepr with
-        | Ocaml.Repr.Field o, Json.Field j -> o, j
-        | _ -> assert false
-      in
-      let ocaml_fname = ocamlf.Ocaml.ocaml_fname in
-      let ocaml_default =
-        match x.f_kind, ocamlf.Ocaml.ocaml_default with
-        | With_default, None ->
-            (match Ocaml.get_implicit_ocaml_default p.deref x.f_value with
-             | None -> Error.error x.f_loc "Missing default field value"
-             | Some d -> Some d
-            )
-        | With_default, Some d -> Some d
-        | Optional, _ -> Some "None"
-        | Required, _ -> None
-      in
-      let json_fname = jsonf.Json.json_fname in
-      let optional = not (Atd.Ast.is_required x.f_kind) in
-      let unwrapped = jsonf.Json.json_unwrapped in
-      (x, ocaml_fname, ocaml_default, json_fname, optional, unwrapped)
-  )
-    (Array.to_list a)
-
 let insert sep l =
   let rec ins sep = function
       [] -> []
@@ -154,7 +109,6 @@ let insert sep l =
     [] -> []
   | x :: l -> x :: ins sep l
 
-
 let make_json_string s = Yojson.Safe.to_string (`String s)
 
 (*
@@ -284,14 +238,20 @@ let write_with_adapter adapter writer =
       let restore =
         Oj_mapping.json_restorer_of_adapter_path adapter_path in
       [
-        Annot ("fun", Line (
+        Line (
           sprintf "Atdgen_runtime.Oj_run.write_with_adapter %s (" restore
-        ));
+        );
         Block writer;
         Line ")";
       ]
 
-let rec make_writer p (x : Oj_mapping.t) : Indent.t list =
+let unwrap_f_value { Ox_emit.mapping; unwrapped; _} (p : param) =
+  if unwrapped then
+    Ocaml.unwrap_option (p.deref mapping.f_value)
+  else
+    mapping.f_value
+
+let rec make_writer ?type_constraint p (x : Oj_mapping.t) : Indent.t list =
   match x with
     Unit _
   | Bool _
@@ -325,8 +285,13 @@ let rec make_writer p (x : Oj_mapping.t) : Indent.t list =
       write_with_adapter adapter standard_writer
 
   | Record (_, a, Record o, Record j) ->
+      let xval =
+        match type_constraint with
+        | Some x -> sprintf "(x : %s)" x
+        | None -> "x"
+      in
       let standard_writer = [
-        Annot ("fun", Line "fun ob x ->");
+        Annot ("fun", Line (sprintf "fun ob %s ->" xval));
         Block (make_record_writer p a o);
       ] in
       let adapter = j.json_record_adapter in
@@ -377,7 +342,7 @@ let rec make_writer p (x : Oj_mapping.t) : Indent.t list =
            ]
 
        | Object ->
-           let k, v = get_assoc_type p.deref loc x in
+           let k, v = Ox_emit.get_assoc_type p.deref loc x in
            let write =
              match o with
                List -> "Atdgen_runtime.Oj_run.write_assoc_list ("
@@ -427,8 +392,8 @@ let rec make_writer p (x : Oj_mapping.t) : Indent.t list =
 and make_variant_writer p ~tick ~open_enum x : Indent.t list =
   let o, j =
     match x.var_arepr, x.var_brepr with
-        Variant o, Variant j -> o, j
-      | _ -> assert false
+      Variant o, Variant j -> o, j
+    | _ -> assert false
   in
   let ocaml_cons = o.Ocaml.ocaml_cons in
   let json_cons = j.Json.json_cons in
@@ -440,8 +405,8 @@ and make_variant_writer p ~tick ~open_enum x : Indent.t list =
       in
       [
         Line (sprintf "| %s%s -> Bi_outbuf.add_string ob %S"
-                 tick ocaml_cons
-                 (enclose (make_json_string json_cons)))
+                tick ocaml_cons
+                (enclose (make_json_string json_cons)))
       ]
   | Some v when open_enum ->
       (* v should resolve to type string. *)
@@ -461,7 +426,7 @@ and make_variant_writer p ~tick ~open_enum x : Indent.t list =
         Line (sprintf "| %s%s x ->" tick ocaml_cons);
         Block [
           Line (sprintf "Bi_outbuf.add_string ob %S;"
-                   (op ^ make_json_string json_cons ^ sep));
+                  (op ^ make_json_string json_cons ^ sep));
           Line "(";
           Block (make_writer p v);
           Line ") ob x;";
@@ -471,7 +436,7 @@ and make_variant_writer p ~tick ~open_enum x : Indent.t list =
 
 and make_record_writer p a record_kind =
   let dot = Ocaml.dot record_kind in
-  let fields = get_fields p a in
+  let fields = Ox_emit.get_fields p.deref a in
   let sep =
     [
       Line "if !is_first then";
@@ -487,11 +452,9 @@ and make_record_writer p a record_kind =
 
   let write_fields =
     List.map (
-      fun (x, ocaml_fname, ocaml_default, json_fname, optional, unwrapped) ->
-        let f_value =
-          if unwrapped then Ocaml.unwrap_option p.deref x.f_value
-          else x.f_value
-        in
+      fun ({ Ox_emit.mapping ; ocaml_fname ; ocaml_default
+           ; json_fname ; optional ; unwrapped } as field) ->
+        let f_value = unwrap_f_value field p in
         let write_field_tag =
           sprintf "Bi_outbuf.add_string ob %S;"
             (make_json_string json_fname ^ ":")
@@ -539,89 +502,83 @@ and make_record_writer p a record_kind =
     Line "Bi_outbuf.add_char ob '}';";
   ]
 
+let wrap_tmp_required_value value =
+  [
+    Line "Some (";
+    Block value;
+    Line ")";
+  ]
+
+let unwrap_tmp_required_value ocaml_fname var =
+  sprintf "(match !%s with Some x -> x \
+           | None -> Atdgen_runtime.Oj_run.missing_field p %S)"
+    var ocaml_fname
+
+(*
+   Json object fields come in an arbitrary order, so we read the fields
+   from the input and keep their value in a temporary slot until we're done
+   reading the object i.e. we find a closing brace '}'. At that point,
+   we must have a value for each field so that we can create an OCaml
+   record.
+
+   Now ATD supports optional fields e.g. `?name: string option;`
+   as well as required fields with a default e.g. `~special : bool;`.
+   These two cases are handled similarly here because in each case we have
+   an initial value to set the field to. For these, we generate essentially
+   the following code:
+
+     let field_x = ref default in
+
+   For the examples above, this is:
+
+     let field_name = ref None in     (* default value for option is None *)
+     let field_special = ref false in (* default value for bool is false *)
+
+   The third case, in which a required field has no default value such as
+   `id : string;`, we don't have a default value so we resort to wrapping
+   in an option, until we're done reading the whole json object.
+   Once the json object has been read, we expect to have found a value for
+   this field:
+
+     let field_id = ref None in   (* no initial value *)
+     ...                          (* read json input *)
+     (* construct the ocaml record: *)
+     {
+       id = (match !field_id with None -> error ... | Some value -> value);
+       ...
+     }
+
+   Note that we no longer use Obj.magic to improve performance because
+   we don't have the resources to do so safely.
+*)
 let study_record ~ocaml_version fields =
   let field_assignments =
     List.fold_right (
-      fun (x, oname, default, jname, opt, unwrap) field_assignments ->
-        let v =
-          match default with
+      fun { Ox_emit.ocaml_fname ; ocaml_default ; optional ; _ }
+        field_assignments ->
+        let var = sprintf "field_%s" ocaml_fname in
+        let initial_value, extract_value =
+          match ocaml_default with
           | None ->
-              assert (not opt);
-              begin match ocaml_version with
-                | Some (maj, min) when (maj > 4 || maj = 4 && min >= 3) ->
-                    "Obj.magic (Sys.opaque_identity 0.0)"
-                | _ -> "Obj.magic 0.0"
-              end
+              assert (not optional);
+              ("None",
+               unwrap_tmp_required_value ocaml_fname var)
           | Some s ->
-              s
+              (s,
+               sprintf "!%s" var)
         in
-        let init = Line (sprintf "let field_%s = ref (%s) in" oname v) in
-        let create = Line (sprintf "%s = !field_%s;" oname oname) in
-        (init, create) :: field_assignments
+        let init_tmp_field =
+          Line (sprintf "let %s = ref (%s) in" var initial_value)
+        in
+        let create_field =
+          Line (sprintf "%s = %s;" ocaml_fname extract_value)
+        in
+        (init_tmp_field, create_field) :: field_assignments
     ) fields []
   in
-  let init_fields, create_record_fields = List.split field_assignments in
-  let n, mapping =
-    List.fold_left (
-      fun (i, acc) (x, oname, default, jname, opt, unwrap) ->
-        if not opt then
-          (i+1, (Some i :: acc))
-        else
-          (i, (None :: acc))
-    ) (0, []) fields
-  in
-  let mapping = Array.of_list (List.rev mapping) in
-
+  let init_tmp_fields, create_record_fields = List.split field_assignments in
   let create_record = [ Line "{"; Block create_record_fields; Line "}" ] in
-
-  let k = n / 31 + (if n mod 31 > 0 then 1 else 0) in
-  let init_bits = List.init k (fun i -> Line (sprintf "let bits%i = ref 0 in" i)) in
-  let final_bits = Array.make k 0 in
-  for z0 = 0 to List.length fields - 1 do
-    match mapping.(z0) with
-        None -> ()
-      | Some z ->
-          let i = z / 31 in
-          let j = z mod 31 in
-          final_bits.(i) <- final_bits.(i) lor (1 lsl j);
-  done;
-  let set_bit z0 =
-    match mapping.(z0) with
-        None -> []
-      | Some z ->
-          let i = z / 31 in
-          let j = z mod 31 in
-          [ Line (sprintf "bits%i := !bits%i lor 0x%x;" i i (1 lsl j)) ]
-  in
-
-  let check_bits =
-    let bool_expr =
-      Array.mapi (fun i x -> sprintf "!bits%i <> 0x%x" i x) final_bits
-      |> Array.to_list
-      |> String.concat " || " in
-    let bit_fields =
-      let a = Array.init k (fun i -> sprintf "!bits%i" i) in
-      sprintf "[| %s |]" (String.concat "; " (Array.to_list a))
-    in
-    let field_names =
-      let l =
-        List.fold_right (
-          fun (x, oname, default, jname, opt, unwrap) acc ->
-            if default = None && not opt then
-              sprintf "%S" x.f_name :: acc
-            else
-              acc
-        ) fields []
-      in
-      sprintf "[| %s |]" (String.concat "; " l)
-    in
-    if k = 0 then []
-    else
-      [ Line (sprintf
-                 "if %s then Atdgen_runtime.Oj_run.missing_fields p %s %s;"
-                 bool_expr bit_fields field_names) ]
-  in
-  init_fields, init_bits, set_bit, check_bits, create_record
+  init_tmp_fields, create_record
 
 let read_with_adapter adapter reader =
   match adapter.Json.ocaml_adapter with
@@ -630,9 +587,9 @@ let read_with_adapter adapter reader =
       let normalize =
         Oj_mapping.json_normalizer_of_adapter_path adapter_path in
       [
-        Annot ("fun", Line (
+        Line (
           sprintf "Atdgen_runtime.Oj_run.read_with_adapter %s (" normalize
-        ));
+        );
         Block reader;
         Line ")";
       ]
@@ -741,7 +698,7 @@ let rec make_reader p type_annot (x : Oj_mapping.t) : Indent.t list =
            ]
 
        | Object ->
-           let k, v = get_assoc_type p.deref loc x in
+           let k, v = Ox_emit.get_assoc_type p.deref loc x in
            let read =
              match o with
                List -> "Atdgen_runtime.Oj_run.read_assoc_list ("
@@ -934,8 +891,8 @@ and make_cases_reader p type_annot ~tick ~open_enum ~std ~fallback_expr l =
 
 and make_record_reader p type_annot loc a json_options =
   let keep_nulls = json_options.json_keep_nulls in
-  let fields = get_fields p a in
-  let init_fields, init_bits, set_bit, check_bits, create_record =
+  let fields = Ox_emit.get_fields p.deref a in
+  let init_fields, create_record =
     study_record ~ocaml_version:p.ocaml_version fields
   in
 
@@ -943,52 +900,50 @@ and make_record_reader p type_annot loc a json_options =
     let a = Array.of_list fields in
     let cases =
       Array.mapi (
-        fun i (x, ocaml_fname, ocaml_default, json_fname, opt, unwrapped) ->
-          let f_value =
-            if unwrapped then Ocaml.unwrap_option p.deref x.f_value
-            else x.f_value
+        fun i ({ Ox_emit.mapping ; ocaml_fname ; json_fname ; optional
+               ; unwrapped ; ocaml_default = _ } as field) ->
+          let f_value = unwrap_f_value field p in
+          let wrap l =
+            if unwrapped then
+              [
+                Line "Some (";
+                Block l;
+                Line ")"
+              ]
+            else l
           in
-        let wrap l =
-          if unwrapped then
+          let read_value =
             [
-              Line "Some (";
-              Block l;
-              Line ")"
+              Line "(";
+              Block (make_reader p None f_value);
+              Line ") p lb";
             ]
-          else l
-        in
-        let read_value =
-          [
-            Line "(";
-            Block (make_reader p None f_value);
-            Line ") p lb";
-          ]
-        in
-        let expr =
-          [
-            Line (sprintf "field_%s := (" ocaml_fname);
-            Block (wrap read_value);
-            Line ");";
-            Inline (set_bit i);
-          ]
-        in
-        let opt_expr =
-          if opt then
-            if keep_nulls then
-              expr
-            else
+          in
+          let read_value =
+            if optional then read_value
+            else wrap_tmp_required_value read_value
+          in
+          let expr =
+            [
+              Line (sprintf "field_%s := (" ocaml_fname);
+              Block (wrap read_value);
+              Line ");";
+            ]
+          in
+          let opt_expr =
+            if optional && not keep_nulls then
               (* treat fields with null values as missing fields
                  (atdgen's default) *)
               [
                 Line "if not (Yojson.Safe.read_null_if_possible p lb) \
-                       then (";
+                      then (";
                 Block expr;
                 Line ")"
               ]
-          else
-            expr
-        in
-        (json_fname, opt_expr)
+            else
+              expr
+          in
+          (json_fname, opt_expr)
       ) a
     in
     let int_mapping_function, int_matching =
@@ -1022,7 +977,6 @@ and make_record_reader p type_annot loc a json_options =
     Line "Yojson.Safe.read_space p lb;";
     Line "Yojson.Safe.read_lcurl p lb;";
     Inline init_fields;
-    Inline init_bits;
     Line "try";
     Block [
       Line "Yojson.Safe.read_space p lb;";
@@ -1040,7 +994,6 @@ and make_record_reader p type_annot loc a json_options =
     Line "with Yojson.End_of_object -> (";
     Block [
       Block [
-        Inline check_bits;
         Line "(";
         Block create_record;
         Line (sprintf "%s)" (Ox_emit.insert_annot type_annot));
@@ -1111,11 +1064,7 @@ and make_tuple_reader p a =
                   Line "x"
                 ]
               else
-                let default_value =
-                  match default with
-                    None -> assert false
-                  | Some s -> s
-                in
+                let default_value = Option.value_exn default in
                 [
                   Line (sprintf "if !end_of_tuple then (%s)" default_value);
                   Line "else (";
@@ -1219,9 +1168,13 @@ let make_ocaml_json_writer p ~original_types is_rec let1 let2 def =
   let param = def.def_param in
   let write = get_left_writer_name p name param in
   let to_string = get_left_to_string_name p name param in
-  let writer_expr = make_writer p x in
-  let eta_expand = is_rec && not (Ox_emit.is_function writer_expr) in
   let needs_annot = Ox_emit.needs_type_annot x in
+  let writer_expr =
+    if needs_annot
+    then make_writer ~type_constraint p x
+    else make_writer p x
+  in
+  let eta_expand = is_rec && not (Ox_emit.is_lambda writer_expr) in
   let extra_param, extra_args, type_annot =
     match eta_expand, needs_annot with
     | true, false -> " ob x", " ob x", None
@@ -1255,7 +1208,7 @@ let make_ocaml_json_reader p ~original_types is_rec let1 let2 def =
     | false -> None
   in
   let reader_expr = make_reader p type_annot x in
-  let eta_expand = is_rec && not (Ox_emit.is_function reader_expr) in
+  let eta_expand = is_rec && not (Ox_emit.is_lambda reader_expr) in
   let extra_param, extra_args =
     if eta_expand then " p lb", " p lb"
     else "", ""
@@ -1284,14 +1237,14 @@ let make_ocaml_json_impl
     ~with_create ~force_defaults ~preprocess_input ~original_types
     ~ocaml_version
     buf deref defs =
-  let p = {
-    deref = deref;
-    std = std;
-    unknown_field_handler = unknown_field_handler;
-    force_defaults = force_defaults;
-    preprocess_input;
-    ocaml_version;
-  } in
+  let p =
+    { deref
+    ; std
+    ; unknown_field_handler
+    ; force_defaults
+    ; preprocess_input
+    ; ocaml_version
+    } in
   defs
   |> List.concat_map (fun (is_rec, l) ->
     let l = List.filter (fun x -> x.def_value <> None) l in
@@ -1387,7 +1340,7 @@ let make_ocaml_files
       Atd.Util.tsort
   in
   let m1 = tsort m0 in
-  let defs1 = Oj_mapping.defs_of_atd_modules m1 in
+  let defs1 = Oj_mapping.defs_of_atd_modules m1 ~target in
   let (m1', original_types) =
     Atd.Expand.expand_module_body ~keep_poly:true m0
   in
@@ -1397,7 +1350,7 @@ let make_ocaml_files
      m2 = monomorphic type definitions after dependency analysis *)
   let ocaml_typedefs =
     Ocaml.ocaml_of_atd ~pp_convs ~target:Json ~type_aliases (head, m1) in
-  let defs = Oj_mapping.defs_of_atd_modules m2 in
+  let defs = Oj_mapping.defs_of_atd_modules m2 ~target in
   let header =
     let src =
       match atd_file with
